; --==Command and Conquer 86==--
;
; David Phillips <electrum@tfs.net>			// Programming
; Dave Scheltema <scheltem@aaps.k12.mi.us>	// Graphics
;
; Started:     09/05/98
; Last Update: 12/10/98
;
;
; Notes:  	I used a unique sort of tripple-buffering system for this game.
;			There are three buffers used:  video ram, the virtual screen and
;			the map screen.  Because the map is not always updated in a frame,
;			it is saved in the map screen.  The sidebar is also stored on the
;			the map screen.  Each frame it is copied over to the virtual screen
;			before any other sprites are drawn.  This is saves time because
;			the background behind sprites doesn't have to be saved and redrawn
;			every frame.  This also helps to equalize the speed, so it doesn't
;			seem to bog down when many sprites are on screen at once (it just
;			runs a little slower all the time).  I'm sure there is a better
;			method, but this seems to work pretty well at the moment.
;
; Units:
;		0 -- Minigunner
;		1 -- Grenadier
;		2 -- Bazooka
;		3 -- Flamethrower
;		4 -- Engineer
;		5 -- Commando
;		6 -- Hum Vee
;		7 -- Recon Bike
;		8 -- Light Tank
;		9 -- Medium Tank
;		A -- Heavy Tank
;		B -- Mammoth Tank
;		C -- Rocket Launcher
;		D -- SSM Launcher
;		E -- Flame Tank
;		F -- Harvester
;
; Buildings:
;		0 -- Construction Yard		3
;		1 -- Power Plant			2
;		2 -- Adv. Power Plant		2
;		3 -- Barracks				2
;		4 -- Refinery				3
;		5 -- Guard Tower			1
;		6 -- Adv. Guard Tower		1
;		7 -- Weapons Factory		3
;		8 -- Comm Center			2
;		9 -- Turret					1
;		A -- Repair Pad				3
;		B -- Obelisk of Light		1
;		C -- Sandbag Wall			1
;		D -- Concrete Wall			1
;		E -- Adv. Comm Center		2
;		F -- Temple of NOD			3
;
;
; Weapon types:
;		* Machine gun
;		* Grendade
;		* Rocket
;		* Flame thrower
;		* Sniper Rifle
;		* Small Shell
;		* Large Shell
;		* Missile Pack
;		* SSM
;		* Laser
;
;
;

#include "ti86asm.inc"

.org _asm_exec_ram


VirtScr 	= _plotSScreen
MapScr		= $f600
VideoRam 	= $fc00
TempRam 	= $800b				; $800b to $bfea = 16352 bytes  (thanks Dux!)


clipmask  	= _textShadow		; [1]	tempory bytes used by the ASCR
rows2put  	= _textShadow + 1	; [1]	...
FSByte		= _textShadow + 2	; [1]	...

MouseX		= _textShadow + 3	; [1]	mouse coordinates
MouseY		= _textShadow + 4	; [1]	...
MouseB		= _textShadow + 5	; [1]	status of the mouse button
MouseBoxS	= _textShadow + 6	; [2]	starting corner of the mouse selection box
MouseBoxE	= _textShadow + 8	; [2]	end corner of the mouse selection box

MapX		= _textShadow + 10	; [1]	upper-left corner of the map display
MapY		= _textShadow + 11	; [1]	...
MapRedraw	= _textShadow + 12	; [1]	flag that means map needs to be redrawn

BListB		= _textShadow + 13	; [15]	building number at each slot in array
BListBNum	= _textShadow + 28	; [1]	number of buildings in array
BListBStart	= _textShadow + 29	; [1]	slot that drawing starts with
BListBCurr	= _textShadow + 30	; [1]	building currently under construction
BListBTime	= _textShadow + 31	; [1]	time left with construction
BListBDraw	= _textShadow + 32	; [1]	flag that means building list needs to be redrawn
BListBKey	= _textShadow + 33	; [1]	flag that means the key has already been pressed

BListU		= _textShadow + 34	; [18]	unit number at each slot in array
BListUNum	= _textShadow + 52	; [1]	number of units in array
BListUStart	= _textShadow + 53	; [1]	slot that drawing starts with
BListUCurr	= _textShadow + 54	; [1]	unit currently under construction
BListUTime	= _textShadow + 55	; [1]	time left with construction
BListUDraw	= _textShadow + 56	; [1]	flag that means unit list needs to be redrawn
BListUKey	= _textShadow + 57	; [1]	flag that means the key has already been pressed

UserStatus	= _textShadow + 58	; [1]   flag that tells what the user's current interface status is
										(0 = nothing, 1 = move, 2 = attack, 3 = place building,
										 4 = launch nuke, 5 = fire ion cannon)

Map			= TempRam			; [4096]	the 64x64 tilemap...


ProgramHeader:
 nop								; for the title line in shells (ASE, Rascall)

 jp Start  							; jump to real start of program

 .dw $0000							; used by shell
 .dw ShellTitle						; pointer to title of the program
 .dw MapSprites-_asm_exec_ram		; relative pointer to map sprites for editor(s)
 .db (MapSpritesEnd-MapSprites)>>3	; number of sprites
ShellTitle:
 .db "Command & Conquer 86",0		; text in the shell title


Start:
 call _flushallmenus				; close any open menus
 call _runindicoff					; turn off the run indicator
 res appTextSave,(iy+1appflags)		; don't write to text shadow
 
ShowTitlePic:
 ld hl,TitlePic				; show the cool title picture
 ld de,$fc00				; display to video memory
 call DispRLE				; uncompress the picture
 ld hl,$2c23				; text location for David's name
 ld (_penCol),hl			; save location
 ld hl,TitleText1			; point to text
 call _vputs				; print it
 ld hl,$3911				; text location for Dave's name
 ld (_penCol),hl			; save location
 ld hl,TitleText2			; point to text
 call _vputs 				; print it
TitleL:
 halt						; wait for interrupt--save batteries
 call _getky				; check for keypresses
 or a						; has a key been pressed?
 jr z,TitleL				; wait for a key to be hit

MainMenu:
 res textwrite,(iy+new_grf_flgs)	; write text to $fc00
 call _clrLCD				; clear the screen
 ld hl,$3911				; small font at the bottom in the center
 ld (_penCol),hl			; save small font position
 ld hl,PlayCountText		; point to first part of the text
 call _vputs				; print it
 ld hl,(PlayCount)			; get play count
 call DisplayHL				; print it
 ld hl,PlayCountText2		; point to second part of the text
 call _vputs				; print it

 ei							; always nice to enable interrupts
 ld hl,MainMenuData			; point to main menu table
 jp DoMenuNoClearLCD		; do the main menu, but don't clear the lcd


SingleStart:
MultiStart:

loaddata:
 ld hl,mapvar-1				; point to var name (first byte is irrelevant)
 rst 20h					; load name into OP1
 rst 10h					; call _FINDSYM
 ret c 						; if it doesn't exist, return
 ld a,b						; copy high byte
 ex de,hl					; swap low bytes
 call $4c3f 				; skip length word (ahl += 2)
 call _set_abs_src_addr		; set the source address as the var
 ld a,1						; copy to ram page 1
 ld hl,Map-$8000			; start at $8000 on page 1 (page 1 is at $10000 ABS)
 call _set_abs_dest_addr	; set the destination as ram page 1
 xor a						; clear high byte
 ld hl,4096					; size of string var
 call _set_mm_num_bytes		; set the number of bytes to copy
 call _mm_ldir				; copy the map
 call _ram_page_1			; make sure ram page 1 is still loaded
 jr dataloaded
mapvar:
 .db 8,"cnc86map"
dataloaded:


InitVars:
 di									; run with un-godly speed (no interrupts!)
 set textwrite,(iy+new_grf_flgs)	; write text to VirtScr

 ld hl,$381c				; start mouse cursor in the center
 ld (MouseX),hl				; set mouse position
 xor a						; start mouse button not pressed
 ld (MouseB),a				; set mouse button

 ld (MapX),a				; set map x coord to 0
 ld (MapY),a				; set map y coord to 0 

 ld (BListBNum),a			; set building build list number to 0
 ld (BListBStart),a			; set building build list start to 0
 ld (BListBDraw),a			; set building build list draw flag to 0
 ld (BListBKey),a			; set building build list key flag to 0

 ld (BListUNum),a			; set unit build list number to 0
 ld (BListUStart),a			; set unit build list start to 0
 ld (BListUDraw),a			; set unit build list draw flag to 0
 ld (BListUKey),a			; set unit build list key flag to 0

 ld a,1
 ld (BListBDraw),a
 ld a,15
 ld (BListBNum),a
 ld b,a
 ld hl,BListB
myloop:
 ld a,16
 sub b
 ld (hl),a
 inc hl
 djnz myloop

 ld a,1
 ld (BListUDraw),a
 ld a,18
 ld (BListUNum),a
 ld b,a
 ld hl,BListU
myloop2:
 ld a,18
 sub b
 ld (hl),a
 inc hl
 djnz myloop2



DrawControls:
 ld hl,ControlPic			; point to control picture
 ld de,MapScr				; map buffer is the target
 call DispRLE				; extract the picture

 call DrawMap				; draw the map for the first time

MainLoop:
 ld a,$FF					; from Matthew Shepcar's Repton
 out (1),a 					; strange things happen w/o this...
 
DumpVirtScr:
 ld hl,VirtScr				; copy the virtual screen
 ld de,VideoRam				; to video ram
 ld bc,1024					; it is 1024 bytes long
 ldir						; dump the screen

 ld a,(MapRedraw)			; get the map redraw flag
 or a						; did we scroll?
 jr z,CopyMapScr			; if not, just copy the screen
 call DrawMap				; we scrolled, redraw the map
 xor a						; need to clear the map redraw flag
 ld (MapRedraw),a			; clear map redraw flag

CopyMapScr:
 ld hl,MapScr				; copy the map screen
 ld de,VirtScr				; to the virtual screen
 ld bc,1024					; it is 1024 bytes long
 ldir						; dump the screen

DoMouse:
 ld bc,(MouseX)				; get the mouse coords
 ld hl,CursorP				; assume pointer cursor
 ld a,%01111110				; bitmask for the arrows
 out (1),a					; tell the port to check
 nop \ nop					; give it time to check
 in a,(1)					; read the keys
 rra						; move last bit to carry flag
 jr nc,DoMouseDown			; bit 0 is for down
 rra						; check the next key
 jr nc,DoMouseLeft			; bit 1 is for left
 rra						; check the next key
 jr nc,DoMouseRight			; bit 2 is for right
 rra						; check the last key
 jr nc,DoMouseUp			; bit 3 is for up
 jr DoMouseS				; we're ready to select the cursor
DoMouseDown:
 ld a,63					; are we at the bottom?
 cp c						; check the y coord
 jr z,DoMouseS				; then we don't want to move it
 inc c						; move the mouse down one
 jr DoMouseS				; ready to pick cursor
DoMouseLeft:
 xor a						; are we at the left?
 cp b						; check the x coord
 jr z,DoMouseS				; then we don't want to move it
 dec b						; move the mouse left one
 jr DoMouseS				; ready to pick cursor
DoMouseRight:
 ld a,127					; are we at the right?
 cp b						; check the x coord
 jr z,DoMouseS				; then we don't want to move it
 inc b						; move the mouse right one
 jr DoMouseS				; ready to pick cursor
DoMouseUp:
 xor a						; are we at the top?
 cp c						; check the y coord
 jr z,DoMouseS				; then we don't want to move it
 dec c						; move the mouse up one [fall through]
DoMouseS:
 ld (MouseX),bc				; save the new coords
 xor a						; check with 0
 cp b						; left edge?
 jr z,DoMouseL				; handle left edge
 cp c						; top edge?
 jr z,DoMouseT				; handle top edge
 ld a,127					; check with 127
 cp b						; right edge?
 jr z,DoMouseR				; handle right edge
 ld a,63					; check with 63
 cp c						; bottom edge?
 jr z,DoMouseB				; handle bottom edge
 jr DoMouseDraw				; we're ready to draw it
DoMouseL:
 ld hl,CursorW				; assume west cursor
 call ScrollLeft			; scroll the screen left
 xor a						; check with 0
 cp c						; top edge also?
 jr nz,DoMouseL2			; skip it
 ld hl,CursorNW				; north west cursor
 call ScrollUp				; scroll the screen up
 jr DoMouseDraw				; we're ready to draw it
DoMouseL2:
 ld a,63					; check with 63
 cp c						; bottom edge also?
 jr nz,DoMouseDraw			; skip it
 ld hl,CursorSW				; south west cursor
 call ScrollDown			; scroll the screen down
 jr DoMouseAdjY				; draw the full cursor
DoMouseT:
 ld hl,CursorN				; assume north cursor
 call ScrollUp				; scroll the screen up
 ld a,127					; check with 127
 cp b						; right edge also?
 jr nz,DoMouseDraw			; skip it
 ld hl,CursorNE				; north east cursor 
 call ScrollRight			; scroll the screen right
 jr DoMouseAdjX				; draw the full cursor 
DoMouseR:
 ld hl,CursorE				; assume east cursor
 call ScrollRight			; scroll the screen right
DoMouseAdjX:
 ld a,b						; get the x coord
 sub 7						; draw the full cursor
 ld b,a						; save new x draw coord 
 ld a,63					; check with 63
 cp c						; bottom edge also?
 jr nz,DoMouseDraw			; skip it
 ld hl,CursorSE				; south east cursor
 call ScrollDown			; scroll the screen down
 jr DoMouseAdjY				; draw the full cursor
DoMouseB:
 ld hl,CursorS				; assume south cursor
 call ScrollDown			; scroll the screen down
DoMouseAdjY:
 ld a,c						; get the y coord
 sub 7						; draw the full cursor
 ld c,a						; save new y draw coord [fall through]
DoMouseDraw:
 call PutSpriteF			; draw the mouse, clip full screen [fall through]

OtherKeys:
 ld a,%00111111				; load the row for 2nd (bit 6 is 0)
 out (1),a					; tell the port to check
 nop \ nop					; give the port time to check the hardware
 in a,(1)					; read the key status
 push af					; save key status
 bit 5,a					; test the bit for the 2nd key
 jr z,OtherKeysM			; handle the mouse
 ld hl,MouseB				; point to mouse button
 xor a						; need to check with 0
 or (hl)					; is it 0?
 jr z,OtherKeys2			; then skip it
 ld (hl),0					; set it to 0
; MOUSE RELEASED HERE
 jr OtherKeys2				; ready to check other keys now
OtherKeysM:
 ld hl,MouseB				; point to mouse button
 xor a						; need to check with 0
 or (hl)					; is it 0?
 jr z,OtherKeysMN			; it wasn't pressed, so skip it
; HELD DOWN HERE
 
 ld hl,(MouseX)				; get mouse position
 ld a,h						; check x coord
 cp 126						; at the right edge?
 jr nc,ClickMapS			; then skip it
 cp 99						; too far left?
 jr c,ClickMapS				; then skip it
 ld a,l						; check y coord
 cp 21						; too far down?
 jr nc,ClickMapS			; then skip it
 cp 2						; at the top edge?
 jr c,ClickMapS				; then skip it
 ld a,h						; get x coord
 sub 99						; map box starts at 99
 rla 						; multiply by 2
 ld (MapX),a				; set new map x coord
 dec l						; the map box
 dec l						; starts at 2
 ld a,l						; get y coord
 rla						; multiply by 2
 add a,l					; multiply by 3
 ld (MapY),a				; set new map y coord  
 ld a,1						; we scrolled (sort of)
 ld (MapRedraw),a			; set map redraw flag to 1
ClickMapS:

 ld hl,(MouseX)				; get mouse position
 ld (MouseBoxE),hl			; save it to box end
 ex de,hl					; put coords into second pair
 ld hl,(MouseBoxS)			; get starting coord
; call Box					; draw the box
 jr OtherKeys2				; ready to check other keys now
OtherKeysMN: 
 ld (hl),1					; set it to pressed
; MOUSE PRESSED HERE
 ld hl,(MouseX)				; get mouse position
 ld (MouseBoxS),hl			; save it to box start
 ld (MouseBoxE),hl			; and save it to box end [fall through]
OtherKeys2:
 pop af						; restore key status
 bit 6,a					; test the bit for the exit key
 jp z,MainMenu				; exit back to the main menu

SideBarBKeys:
 ld a,%01011111				; check for Alpha, Graph
 out (1),a					; tell the port to check
 nop \ nop					; wait for the port
 in a,(1)					; read key status from port
 rla						; move Alpha status to carry
 jr nc,SideBarBUp			; handle scroll up key
 rla						; move Graph status to carry
 jr c,SideBarBDraw			; if not, don't handle scroll down key
SideBarBDown:
 ld hl,BListBKey			; point to key flag
 xor a						; need to check with 0
 or (hl)					; is it clear?
 jr nz,SideBarUKeys			; if not, skip everything
 inc (hl)					; set it
 ld a,(BListBNum)			; get number of items in build list
 ld hl,BListBStart			; point to start indicator
 sub (hl)					; find distance
 cp 5						; are we at the bottom?
 jr c,SideBarBDraw			; then we can't scroll
 inc (hl)					; scroll down
 jr SideBarBDrawY			; draw the sidebar 
SideBarBUp:
 ld hl,BListBKey			; point to key flag
 xor a						; need to check with 0
 or (hl)					; is it clear?
 jr nz,SideBarUKeys			; if not, skip everything
 inc (hl)					; set it
 ld hl,BListBStart			; point to start indicator
 ld a,(hl)					; get the value
 or a						; are we at the top?
 jr z,SideBarBDraw			; then we can't scroll
 dec (hl)					; scroll up
 jr SideBarBDrawY			; draw the sidebar
SideBarBDraw:
 ld hl,BListBDraw			; point to the draw flag
 xor a						; need to check with 0
 ld (BListBKey),a			; clear key flag now, while A is 0
 or (hl)					; is it clear?
 jr z,SideBarUKeys			; then don't draw it
 dec (hl) 					; set it to 0
SideBarBDrawY:
 ld hl,BListB				; point to the build list
 ld d,0						; clear upper byte
 ld a,(BListBStart)			; load the start number
 ld e,a						; load start number into the lower byte
 add hl,de					; start at correct position in list
 ld a,(BListBNum)			; get number of items
 sub e						; compute number of items from bottom
 jr z,SideBarUKeys			; if it's empty, don't draw anything
 cp 5						; more than 4?
 jr c,SideBarBDrawS			; if not, skip change
 ld a,4						; only draw 4
SideBarBDrawS:
 ld b,a						; setup loop counter with number
 ld de,(3<<8)+12			; start drawing at 3,12
SideBarBDrawL:
 push hl					; save pointer
 push de					; save coords
 push bc					; save loop counter
 ld b,0						; clear high byte
 ld c,(hl)					; get sprite number
 sla c						; multiply by 2
 sla c						; by 4
 sla c						; by 8, the size of sprites
 ld hl,SideBarB_0			; point to sidebar sprites
 add hl,bc					; find correct sprite to draw
 call GridPutSprite			; finally we can draw the sprite
 pop bc						; restore loop counter
 pop de						; restore coords
 pop hl						; restore pointer
 inc d						; move down a sprite
 inc hl						; next item
 djnz SideBarBDrawL			; draw all the sprites

SideBarUKeys:
 ld a,%01101111				; check for x-Var, Table
 out (1),a					; tell the port to check
 nop \ nop					; wait for the port
 in a,(1)					; read key status from port
 rla						; move x-Var key status to carry
 jr nc,SideBarUUp			; handle scroll up key
 rla						; move Table status to carry
 jr c,SideBarUDraw			; if not, don't handle scroll down key
SideBarUDown:
 ld hl,BListUKey			; point to key flag
 xor a						; need to check with 0
 or (hl)					; is it clear?
 jr nz,DoRadar				; if not, skip everything
 inc (hl)					; set it
 ld a,(BListUNum)			; get number of items in build list
 ld hl,BListUStart			; point to start indicator
 sub (hl)					; find distance
 cp 5						; are we at the bottom?
 jr c,SideBarUDraw			; then we can't scroll
 inc (hl)					; scroll down
 jr SideBarUDrawY			; draw the sidebar 
SideBarUUp:
 ld hl,BListUKey			; point to key flag
 xor a						; need to check with 0
 or (hl)					; is it clear?
 jr nz,DoRadar				; if not, skip everything
 inc (hl)					; set it
 ld hl,BListUStart			; point to start indicator
 ld a,(hl)					; get the value
 or a						; are we at the top?
 jr z,SideBarUDraw			; then we can't scroll
 dec (hl)					; scroll up
 jr SideBarUDrawY			; draw the sidebar
SideBarUDraw:
 ld hl,BListUDraw			; point to the draw flag
 xor a						; need to check with 0
 ld (BListUKey),a			; clear key flag now, while A is 0
 or (hl)					; is it clear?
 jr z,DoRadar				; then don't draw it
 dec (hl) 					; set it to 0
SideBarUDrawY:
 ld hl,BListU				; point to the build list
 ld d,0						; clear upper byte
 ld a,(BListUStart)			; load the start number
 ld e,a						; load start number into the lower byte
 add hl,de					; start at correct position in list
 ld a,(BListUNum)			; get number of items
 sub e						; compute number of items from bottom
 jr z,DoRadar				; if it's empty, don't draw anything
 cp 5						; more than 4?
 jr c,SideBarUDrawS			; if not, skip change
 ld a,4						; only draw 4
SideBarUDrawS:
 ld b,a						; setup loop counter with number
 ld de,(3<<8)+14			; start drawing at 3,14
SideBarUDrawL:
 push hl					; save pointer
 push de					; save coords
 push bc					; save loop counter
 ld b,0						; clear high byte
 ld c,(hl)					; get sprite number
 sla c						; multiply by 2
 sla c						; by 4
 sla c						; by 8, the size of sprites
 ld hl,SideBarU_0			; point to sidebar sprites
 add hl,bc					; find correct sprite to draw
 call GridPutSprite			; finally we can draw the sprite
 pop bc						; restore loop counter
 pop de						; restore coords
 pop hl						; restore pointer
 inc d						; move down a sprite
 inc hl						; next item
 djnz SideBarUDrawL			; draw all the sprites

DoRadar:
 ld a,(MapY)				; load y coord
 ld b,3						; need to divide by 3
 ld c,0						; it starts at 1, always add once
DoRadarL:					; note: this loop will almost always be faster than Divide 
 inc c						; increase quotient
 sub b						; subtract divisor
 jr nc,DoRadarL				; loop until it's negative
 ld a,(MapX)				; load x coord
 sra a						; divide by 2
 add a,97					; it starts at 97
 ld b,a						; save x coord for draw
 ld hl,RadarBox				; point to radar sprite
 call PutSpriteF			; draw it

EndMainLoop:
 jp MainLoop				; time to go back to the top of main loop

Exit:
 set appAutoScroll,(iy+appflags)	; turn scrolling back on
 set appTextSave,(iy+appflags)		; need to clear the text shadow
 call _clrScrn						; clear the display and text shadow
 call _homeup						; move cursor to upper-left corner
 
 ld hl,(PlayCount)					; get play count
 inc hl								; increase it
 ld (PlayCount),hl					; now save it

SaveSettings:
 ld hl,ProgName						; point to program name
 rst 20h							; load into OP1 with _MOV10TOOP1
 rst 10h							; find it with _FINDSYM
 ex de,hl							; point to start of variable
 ld a,b								; get top of absolute pointer
 ld de,SettingData-$d748+4			; relative offset - 4
 add hl,de							; form pointer to var
 adc a,0							; add carry
 ld de,SettingData					; point to data in _asm_exec_ram
 ld b,SettingDataEnd-SettingData	; copy entire length of SettingData
SaveSettingsL:
 call _getb_ahl						; convert abs pointer
 ld a,(de)							; read value
 ld (hl),a							; save new value to variable
 call _inc_ptr_ahl					; move to next destination byte
 djnz SaveSettingsL					; loop for all bytes
 
 ret 								; return to the TI-OS or a shell
									; ...and hope it's not Iridus or AShell

Help:
 ld hl,HelpText				; point to the help text
 call TextView				; view the text
 jp MainMenu				; go back to the main menu

Options:
 call _clrLCD
 call _homeup
 ld hl,OptionText
 call _puts
OptionsL:
 halt						; wait for interrupt--save batteries
 call _getky				; check for keypresses
 or a						; has a key been pressed?
 jr z,OptionsL				; wait for a key to be hit
 jp MainMenu				; go back to the main menu
OptionText:
.db "Many options here!",0


PutSpriteF:
 ld a,127-6					; clip entire screen (127)
 ld (PSC_Modify+1),a		; self-modify PutSprite's clipping
 call PutSprite				; draw the sprite
 ld a,96-6					; clip map area (96)
 ld (PSC_Modify+1),a		; self-modify PutSprite's clipping
 ret						; done

ScrollUp:
 push hl					; save hl
 ld hl,MapY					; point to map y coord
 xor a						; check with 0
 cp (hl)					; are we at the top?
 jr z,ScrolledD				; then we're done
 dec (hl)					; scroll up
 jr Scrolled				; we're done scrolling

ScrollDown:
 push hl					; save hl
 ld hl,MapY					; point to map y coord
 ld a,56					; check with 56
 cp (hl)					; are we at the bottom?
 jr z,ScrolledD				; then we're done
 inc (hl)					; scroll down
 jr Scrolled				; we're done scrolling

ScrollLeft:
 push hl					; save hl
 ld hl,MapX					; point to map x coord
 xor a						; check with 0
 cp (hl)					; are we at the left side?
 jr z,ScrolledD				; then we're done
 dec (hl)					; scroll left 
 jr Scrolled				; we're done scrolling

ScrollRight:
 push hl					; save hl
 ld hl,MapX					; point to map x coord
 ld a,52					; check with 52
 cp (hl)					; are we at the right side?
 jr z,ScrolledD				; then we're done
 inc (hl)					; scroll right [fall through]

Scrolled:
 ld a,1						; we scrolled
 ld (MapRedraw),a			; set map redraw flag accordingly
ScrolledD:
 pop hl						; restore hl
 ret						; we're done

DrawMap:
 ld de,(MapX)				; load the map coords	(note:  loaded in reverse order!)
 ld h,d						; move the y coord to the high byte (* 256)
 srl h						; divide by two (* 128)
 rra						; move overflow to low byte
 srl h						; again, divide by two (* 64)
 rra						; handle final overflow
 and %11000000				; clear lower 6 bits
 or e						; add the x coord (x <= 63)
 ld l,a						; move to low byte
 ld de,Map					; point to the map
 add hl,de					; add the map start to the offset
 ld b,8						; we need to draw 8 rows
 ld d,0						; start y draw coord at 0
DMYLoop:
 push bc					; save the y counter
 ld b,12					; each row has 12 tiles in it
 ld e,0						; start x draw coord at 0
DMXLoop:
 push bc					; save the x counter
 push hl					; save the map pointer
 push de					; save the tile draw coords
 ld l,(hl)					; load the tile number into the low byte
 ld h,0						; clear the high byte
 add hl,hl					; * 2
 add hl,hl					; * 4
 add hl,hl					; * 8, each sprite is 8 bytes
 ld de,MapSprites 			; point to the start of the map sprites
 add hl,de					; point to the correct sprite
 pop de						; restore the tile draw coords
 push de					; save the tile draw coords
 call GridPutSprite			; draw the sprite!
 pop de						; restore the tile draw coords
 pop hl						; restore the map pointer     
 inc hl						; move to the next tile
 inc e						; move to next x position
 pop bc						; restore the x counter
 djnz DMXLoop				; loop for all tiles in the row
 ld bc,64-12				; each row is 64 bytes wide (12 on the screen)
 add hl,bc					; move to next row
 inc d						; move to next y position
 pop bc						; restore the y counter
 djnz DMYLoop				; draw all the rows
 ret						; finally, and many t-states later, we're done!

;====================================================================
; DisplayHL:  (idea from SCaBBy/Mardell)    [Assembly Coder's Zenith]
;  Display HL as a 5-digit decimal number with no leading zeros
;  out: AF, HL, BC, DE destroyed
;====================================================================
DisplayHL:
 ld c,'0'  							; save ascii value for zero
 ld de,_OP1+5						; point to end of the buffer
 xor a								; zero terminate string
 ld (de),a							; set last byte to zero
DisplayHLl:
 call _divHLby10					; next digit
 dec e								; next position (e > 6)
 add a,c							; convert to ascii
 ld (de),a							; save digit
 ld a,h								; load upper byte
 or l								; check with lower byte for zero
 jr nz,DisplayHLl					; loop
 ex de,hl							; point to buffer
 jp _vputs							; print number and return

;=====================================================================
; GridPutSprite:
;  Puts an aligned sprite at (E, D), where HL -> Sprite
;=====================================================================
; IN : D  = y (0-7 inclusive)
;      E  = x (0-15 inclusive)
;      HL-> sprite
;
; OUT: AF, BC, DE, HL, IX modified
; Current total : 28b/567t (not counting ret)
;=====================================================================
GridPutSprite:
 push hl
 pop ix          	; ix-> sprite
 srl d           	; de = 128y+x (16 bytes/row, 8 rows)
 rra
 and $80
 or e            	; add x offset (remember x <= 15)
 ld e,a
 ld hl,MapScr   	; hl-> MapScr
 add hl,de       	; hl-> MapScr + offset
 ld b,8          	; initialize loop counter
 ld de,$10		 	; initialize line increment
GPS_Loop
 ld a,(ix+0)     	; get byte from sprite
 ld (hl),a      	; put byte on screen
 inc ix				; move to next byte in sprite
 add hl,de			; move to next line on screen
 djnz GPS_Loop
 ret

;===========================================================
; RLE picture displayer v1.1
; Decodes a RLE picture made by RLE2PIC
;
; written by David Phillips <electrum@tfs.net>
; started: 8/19/98
; last update: 11/5/98
;
; input: HL = RLE encoded picture, DE = where to display
; output: 1024 byte decoded picture
; destroys: AF, BC, DE, HL
; current size: 32 bytes
;===========================================================
DispRLE:
 ld bc,1024			; we need to copy 
DispRLEL:
 ld a,(hl)			; get the next byte
 cp $91				; is it a run?
 jr z,DispRLERun	; then we need to decode the run
 ldi				; copy the byte, and update counters
DispRLEC:
 ld a,b				; check the low byte and
 or c				; the high byte for 0
 jr nz,DispRLEL		; if not, then we're not done either
 ret				; if it's zero, we're done
DispRLERun:
 inc hl				; move to the run value
 ld a,(hl)			; get the run value
 inc hl				; move to the run count
 push hl			; save source pointer
 ld h,(hl)			; get the run count
 ex de,hl			; swap source and destination pointers
DispRLERunL:
 ld (hl),a			; copy the byte
 inc hl				; increase destination pointer
 dec bc				; decrease byte count
 dec d				; decrease run count
 jr nz,DispRLERunL	; if we're not done, then loop
 ex de,hl			; swap pointers back
 pop hl				; recover source pointer
 inc hl				; advance the source pointer
 jr DispRLEC		; check to see if we should loop

;=======================================================
; TextView by David Phillips   [Assembly Coder's Zenith]
;
; input: HL = text to display
; format: -must start and end with $ed
;	      -first string must start with a zero-byte
;         -all strings must be at least once char
;          long and be separated with a zero-byte,
;          including the _first_ string
;         -text must have at least 8 strings
;         -max length of a string is 21 chars
;
; example:
;  .db $ed,0					; must start like this
;  .db "Some cool text that",0
;  .db "will be scrolled!",0
;  .db " ",0					; blank line
;  .db "And now the end...",0
;  .db $ed						; must end like this
;=======================================================
TextView:
 res appAutoScroll,(iy+appflags)
 push hl
 call _clrLCD
 pop hl
 inc hl
 inc hl
 ld (TextTop),hl
 ld b,8
 ld d,0
TextViewDL:
 ld a,8
 sub b
 ld e,a
 ld (_curRow),de
 call _puts
 djnz TextViewDL
 ld (TextBottom),hl
TextViewLoop:
 halt					;\ 
 call _getky			; \
 cp K_DOWN				;  \
 jr z,TextViewSUp		;   \
 cp K_UP				;	 >    GET_KEY -- a nice, good speed
 jr z,TextViewSDown 	;   /
 cp K_EXIT				;  /
 jr nz,TextViewLoop		; /
; ld a,%01111110		; \
; out (1),a				;  \
; nop \ nop				;   \
; in a,(1)				;    \
; bit 3,a				;     \
; jr z,TextViewSDown	;	   \
; bit 0,a				;       \ Ports -- if you want it really,
; jr z,TextViewSUp		;       /          really, really fast
; ld a,%00111111		;      /
; out (1),a				;     /
; nop \ nop				;    /
; in a,(1)				;   /
; bit 6,a				;  /
; jr nz,TextViewLoop	; /
 ret
TextViewSUp:
 ld hl,(TextBottom)
 ld a,$ed
 cp (hl)
 jr z,TextViewLoop
 ld hl,$fc00+128
 ld de,$fc00
 ld bc,1024-128
 ldir
 ld hl,$fc00+(128*7)
 ld b,128
 call $437b				; _ldhlz
 ld hl,$0007
 ld (_curRow),hl
 ld hl,(TextBottom)
 call _puts
 xor a
 cp (hl)
 jr nz,TextViewSkipI
 inc hl
TextViewSkipI:
 ld (TextBottom),hl
 ld hl,(TextTop)
 call TextViewPtrI
 ld (TextTop),hl
 jr TextViewLoop
TextViewSDown:
 ld hl,(TextTop)
 dec hl
 dec hl
 ld a,$ed
 cp (hl)
 jr z,TextViewLoop
 ld hl,$ffff-128
 ld de,$ffff
 ld bc,1024-128
 lddr 
 ld hl,$fc00
 ld b,128
 call $437b				; _ldhlz
 call _homeup
 ld hl,(TextTop)
 call TextViewPtrD
 call _puts
 ld hl,(TextTop)
 call TextViewPtrD
 ld (TextTop),hl
 ld hl,(TextBottom)
 call TextViewPtrD
 ld (TextBottom),hl 
 jp TextViewLoop
TextViewPtrD:
 xor a
 ld b,255
 dec hl
 dec hl
 cpdr
 inc hl
 inc hl
 ret
TextViewPtrI:
 xor a
 ld b,255
 cpir
 ret
TextTop = _OP1
TextBottom = _OP1 + 2

;===========================================================
; DoMenu -- Bar menu routine        [Assembly Coders Zenith]
; Clears the screen and displays a table driven bar menu
;  
; written by David Phillips <electrum@tfs.net>
; helped by David Scheltema <scheltem@aaps.k12.mi.us>
; optimizations by CLEM <clems@mygale.org>
;
; input: HL = menu table
; output: chosen table address jumped to
;
; example table:
;  .db 2				; num entries
;  .db "My menu",0		; title text (at the top)
;  .db "First thing",0	; text for each entry
;  .db "Second one",0	; ...
;  .dw Option1			; pointers (labels) to jump
;  .dw Option2			; to for each entry in menu
;
; Note: routine must be jumped to!
;===========================================================
DoMenu:
 push hl					; _clrLCD destroys hl
 call _clrLCD				; clears the screen
 pop hl						; now restore it
DoMenuNoClearLCD:
 ld c,(hl)					; get number of entries
 inc hl						; move to menu title text
 ld de,$0001				; draw at top, one char right
 ld	(_penCol),de			; set the small font coords
 call _vputs				; print the title text
 ld de,$fc00				; point to video ram
 call DoMenuInv				; invert the top line
 ld b,c						; load num entries for loop
 ld de,$0729				; start printing at middle of second line
DoMenuDTL:
 ld (_penCol),de			; set the font coords
 call _vputs				; print the menu text
 ld a,d						; get the vertical coord
 add a,7					; move to next line
 ld d,a						; save new coord
 djnz DoMenuDTL				; draw all the text
 push hl 					; save pointer
DoMenuFirst:
 ld b,1						; start menu at first entry
DoMenuBar:
 call DoMenuInvBar			; invert it
DoMenuL:
 call $4068					; same as GET_KEY, doesn't destroy bc !!!
 cp K_UP					; did they press up?
 jr z,DoMenuUp				; move menu bar up
 cp K_DOWN					; or was it down?
 jr z,DoMenuDown			; then move the bar down
 cp K_SECOND				; did they hit 2nd?
 jr z,DoMenuSelect			; then they want it
 cp K_ENTER					; some people like enter
 jr nz,DoMenuL				; they're just sitting there, loop again
DoMenuSelect:
 pop hl						; restore the pointer
 ld d,0						; clear upper byte
 ld e,b						; load lower byte with current selection
 dec e						; decrease, starting at 0 instead of 1
 sla e						; multiply by 2, pointers are 16-bit (2 bytes)
 add hl,de					; add to get correct address in the jump table
 ld a,(hl)					; get lower byte of pointer
 inc hl						; next byte
 ld h,(hl)					; get upper byte of pointer
 ld l,a						; complete pointer with lower byte
 jp (hl)					; jump to the address from the table--hl, not (hl)
DoMenuUp:
 call DoMenuInvBar			; invert current selection back to normal
 dec b						; move up
 jr nz,DoMenuBar			; if it's zero, we're at the top
 ld b,c						; so move to the bottom
 jr DoMenuBar				; handle menu bar
DoMenuDown:
 call DoMenuInvBar			; same as up, go back to normal text
 ld a,c						; get current entry for check
 cp b						; at the bottom?
 jr z,DoMenuFirst			; the move to the top
 inc b						; move down
 jr DoMenuBar				; handle menu bar
DoMenuInvBar:
 push hl					; save pointer
 push bc					; save position and entries
 ld h,b						; get position
 ld l,112					; there are 7 lines of 16 bytes each (7*16 = 112)
 call _HtimesL				; do the multiply
 ld de,$fc00				; we need to add to video memory
 add hl,de					; do the add
 ex de,hl					; hl -> de
 call DoMenuInv				; invert the line
 pop bc						; restore position and entries
 pop hl						; restore pointer
 ret						; done inverting
DoMenuInv:
 ld b,7*16					; each row is 16 bytes wide and text is 7 rows high
DoMenuIL:
 ld a,(de)					; get the byte
 cpl						; NOT the byte--flips all the bits
 ld (de),a					; save the new byte
 inc de						; next byte
 djnz DoMenuIL				; loop for all bytes
 ret						; we're done with the invert loop

;======================================================
; ReceiveByte                 [Assembly Coder's Zenith]
;  Originally by Pascal Bouron and Jimmy Mardell
;
; out: A = received byte    carry set = error
; destroyed: all registers, OP1
;======================================================
ReceiveByte:
 ld hl,0
 ld (_OP1),hl
 ld 	 e,1			 ; for the OR
 ld 	 c,0			 ; byte receive
 ld 	 b,8			 ; counter
 ld 	 a,$C0
 out	 (7),a
rb_w_Start:
 in 	 a,(7)
 and 3
 cp  3
 jr 	 nz,rb_get_bit
 call rbTest_ON
 jr 	 rb_w_Start
rb_get_bit:
 cp 	 2
 jr 	 z,rb_receive_zero
 ld 	 a,c
 or 	 e
 ld 	 c,a
 ld 	 a,$D4
 out	 (7),a
 jr 	 rb_waitStop
rb_receive_zero:
 ld 	 a,$E8
 out	 (7),a
rb_waitStop:
 call rbTest_ON
 in 	 a,(7)
 and	 3
 jr 	 z,rb_waitStop
 ld 	 a,$c0
 out	 (7),a
 rl 	 e
 djnz 	 rb_w_Start
 ld 	 a,c
 ret
rbTest_ON:
 ld a,(_OP1)
 inc a
 ld (_OP1),a
 cp 255
 ret nz
 pop hl  ;Back to the place you were before.	Gotta love it!
 xor a
 ret

;======================================================
; SendByte                 	  [Assembly Coder's Zenith]
;  Originally by Pascal Bouron and Jimmy Mardell
;
; in: A = byte to send
; out: carry set = error
; destroyed: all registers, OP1
;======================================================
SendByte:
 ld hl,0
 ld (_OP1),hl
 ld 	 b,8
 ld 	 c,a			 ;byte to send
 ld 	 a,$C0
 out	 (7),a
sbw_setport3:
 in 	 a,(7)
 and 3
 cp  3
 jr 	 z,sbcalc_bit
 call sbSendTest_ON
 jr 	 sbw_setport3
sbcalc_bit:
 ld 	 a,c
 and	 1
 jr 	 z,sbsend_one
sbsend_zero:
 ld 	 a,$E8
 out	 (7),A
 jr 	 sbwait_setport
sbsend_one:
 ld 	 a,$D4
 out	 (7),A
sbwait_setport:
 call sbSendTest_ON
 in 	 a,(7)
 and	 3
 jr 	 nz,sbwait_setport
 ld 	 a,$C0
 out	 (7),A
 srl c
 djnz sbw_setport3
 xor a
 ret
sbSendTest_ON:
 ld a,%00111111
 out (1),a
 nop
 nop
 in a,(1)
 bit 6,a
 ret nz
 pop hl
; pop hl		; instead of returning, jump to
; jp Quit		; some program exit code
 ret

;=========================================
; Divide       (d = d / c  ...  a = d % c)
;  by Joshua Grams <j3grams@earthling.net>
; input:  d = dividend, c = divisor
; output: d = quotient, a = remainder
; total: 15b/355t to 363t (exluding RET)
;=========================================
Divide:
 push bc		; save bc
 sub a 			; clear a
 ld b,8 		; 8 shifts
DivideL:
 sla d 			; shift left d
 rla 			; into a
 cp c 			; is a >= c
 jr c,DivideS	; skip if not
 sub c
 inc d			; a = a-c, set bit in d
DivideS:
 djnz DivideL 	; loop
 pop bc			; restore bc
 ret			; done

;
; FASTLINE 1.0
;
; BRESENHAM'S LINE ALGORITHM
; by Jimmy Mrdell <yarin@acc.umu.se>
;
; Parameters:  BC    = memory area to draw line (usually $FC00)
;              DE,HL = coordinates of line (D,E)-(H,L)
;                      0 < D,H < 128, 0 < E,L < 64
;
; Code size:   145 bytes (excluding FindPixel)
;             
; Registers:   AF destroyed.
;
; Performance: 4 times faster than Stephane Jantzens line routine
;
;              Delta_X > Delta_Y  =>  innerloop =  98.50 T states
;              Delta_Y > Delta_X  =>  innerloop = 106.75 T states
;

FLine:
 push bc
 push de
 push hl
 push bc
 ld a,h
 cp d
 jr nc,DXpos
 ex de,hl
DXpos:
 ld b,d
 ld c,e
 pop de
 push hl
 call FIND_PIXEL
 add hl,de
 ex (sp),hl
 ld e,c
 ld c,a
 ld a,h
 sub b
 ld b,a
 ld a,l
 sub e
 ld de,16
 jr nc,DYpos
 neg
 ld de,-16
DYpos:
 push af
 sub b
 jr nc,DY_Greater
 add a,a
 neg
 ld (x_ai+1),a
 pop af
 add a,a
 ld (XDNeg+1),a
 sub b
 pop hl
 push af
 ld a,c
 or (hl)
 ld (hl),a
 pop af
 bit 7,a
 jr nz,XDNeg
XNewRow:
 add hl,de
x_ai:
 sub 0
 push af
 rrc c
 jr nc,XNoWrap1
 inc hl
XNoWrap1:
 ld a,c
 or (hl)
 ld (hl),a
 pop af
 dec b
 jr z,LineDone
 jr nc,XNewRow
XDNeg:
 add a,0
 push af
 rrc c
 jr nc,XNoWrap2
 inc hl
XNoWrap2:
 ld a,c
 or (hl)
 ld (hl),a
 pop af
 dec b
 jr z,LineDone
 jr c,XNewRow
 jr XDNeg
DY_Greater:
 neg
 add a,a
 ld (YNoWrap+1),a
 ld a,b
 add a,a
 ld (YDNeg+1),a
 pop hl
 sub h
 ld b,h
 pop hl
 push af
 ld a,c
 or (hl)
 ld (hl),a
 pop af
YRepeat:
 bit 7,a
 jr nz,YDNeg
 rrc c
 jr nc,YNoWrap
 inc hl
YNoWrap:
 add a,0
 jr YPlot
YDNeg:
 add a,0
YPlot:
 add hl,de
 push af
 ld a,c
 or (hl)
 ld (hl),a
 pop af
 djnz YRepeat
LineDone:
 pop hl
 pop de
 pop bc
 ret

; ASCR	- Advanced Sprite Clipping Routines
;
; 	by Jimmy Mrdell	970304	 Last update 970803
;
; Temporary variables needed:
;  clipmask  : Byte
;  rows2put  : Byte
;
;
; PutSprite:
;  Puts an 8x8 sprite at B,C. HL = pointer to sprite. No mask and no
;  background storage, just clipping.
;
; PutSprite  -- VirtScr, clips (0, 0)-(103, 63)

PutSprite:		 ; BC = x,y  HL = sprite
 push bc
 push de
 push hl
 push ix
 ld a,$FF
 ld (clipmask),a
 bit 7,b
 jr z,PSC_CheckRightClip
 ld a,b
 cp 249
 jr c,EndPPS
 neg
 push bc
 ld b,a
 ld a,$FF
PSC_LeftClip:
 srl a
 djnz PSC_LeftClip
 pop bc
 res 7,b
 dec c
 ld (clipmask),a
 jr PSC_CheckBotClip
PSC_CheckRightClip
 ld a,b
PSC_Modify:					; self modifying code--changes clipping on right side
 sub 96-6
 jr c,PSC_CheckBotClip
 push bc
 ld b,a
 inc b
 ld a,$FF
PSC_RightClip:
 add a,a
 djnz PSC_RightClip
 ld (clipmask),a
 pop bc
PSC_CheckBotClip:
 ld a,8
 ld (rows2put),a
 bit 7,c
 jr nz,PSC_CheckTopClip
 bit 6,c
 jr nz,EndPS
 ld a,64
 sub c
 cp 8
 jr nc,PSC_ClippingDone
 ld (rows2put),a
 jr PSC_ClippingDone
PSC_CheckTopClip:
 ld a,c
 cp 249
EndPPS:
 jr c,EndPS
 push bc
 neg
 ld b,a
 sub 8
 neg
 ld (rows2put),a
PSC_TopClip:
 inc hl
 inc de
 djnz PSC_TopClip
 pop bc
 ld c,0
PSC_ClippingDone:
 push hl
 pop ix
 ld a,(rows2put)
 push af
 call FIND_PIXEL
 ld de,VirtScr
 add hl,de
 ld d,a
 pop bc
PSC_PutRow:
 push bc
 push hl
 ld a,(clipmask)
 ld e,a
 ld b,8
 ld c,(ix)
 inc ix
PSC_PutCol:
 push bc
 rlc e
 jr nc,PSC_NextBit
 ld a,d
 rlc c
 jr nc,PSC_NextBit			; changed so sprites are transparent
; jr c,PSC_BitOn
; cpl
; and (hl)
; ld (hl),a
; jr PSC_NextBit
PSC_BitOn:
 or (hl)
 ld (hl),a
PSC_NextBit:
 rrc d
 jr nc,PSC_SSB
 inc hl
PSC_SSB:
 pop bc
 rlc c
 djnz PSC_PutCol
 pop hl
 ld bc,16
 add hl,bc
 pop bc
 djnz PSC_PutRow
EndPS:
 pop ix
 pop hl
 pop de
 pop bc
 ret

FIND_PIXEL:
 push bc
 push de
 ld hl,ExpTable+1
 ld d,0
 ld a,b
 and $07
 ld e,a
 add hl,de
 ld e,(hl)
 ld h,d
 srl b
 srl b
 srl b
 ld a,c
 add a,a
 add a,a
 ld l,a
 add hl,hl
 add hl,hl
 ld a,e
 ld e,b
 add hl,de
 pop de
 pop bc
 ret

ExpTable:
 .db $01,$80,$40,$20,$10,$08,$04,$02,$01

; Buildings

BuildingSize:
 .db 3,2,2,2,3,1,1,3,2,1,3,1,1,1,2,3


; Sprites

CursorP:				; cursor - pointer
	.db %11000000
	.db %11100000
	.db %11110000
	.db %11111000
	.db %11111100
	.db %11110000
	.db %10111000
	.db %00011000

CursorM:				; cursor - move
	.db %00000000
	.db %00011000
	.db %00000000
	.db %01011010
	.db %01011010
	.db %00000000
	.db %00011000
	.db %00000000

CursorA:				; cursor - attack
	.db %00000000
	.db %00100100
	.db %01100110
	.db %00011000
	.db %00011000
	.db %01100110
	.db %00100100
	.db %00000000

CursorN:				; cursor - north
	.db %00011000
	.db %00111100
	.db %01111110
	.db %11111111
	.db %00011000
	.db %00011000
	.db %00011000
	.db %00000000

CursorE:				; cursor - east
	.db %00001000
	.db %00001100
	.db %00001110
	.db %01111111
	.db %01111111
	.db %00001110
	.db %00001100
	.db %00001000

CursorS:				; cursor - south
	.db %00000000
	.db %00011000
	.db %00011000
	.db %00011000
	.db %11111111
	.db %01111110
	.db %00111100
	.db %00011000

CursorW:				; cursor - west
	.db %00010000
	.db %00110000
	.db %01110000
	.db %11111110
	.db %11111110
	.db %01110000
	.db %00110000
	.db %00010000

CursorNW:				; cursor - north west
	.db %11111100
	.db %11111000
	.db %11110000
	.db %11111000
	.db %11011100
	.db %10001110
	.db %00000100
	.db %00000000

CursorNE:				; cursor - north east
	.db %00111111
	.db %00011111
	.db %00001111
	.db %00011111
	.db %00111011
	.db %01110001
	.db %00100000
	.db %00000000

CursorSE:				; cursor - south east
	.db %00000000
	.db %00100000
	.db %01110001
	.db %00111011
	.db %00011111
	.db %00001111
	.db %00011111
	.db %00111111

CursorSW:				; cursor - south west
	.db %00000000
	.db %00000100
	.db %10001110
	.db %11011100
	.db %11111000
	.db %11110000
	.db %11111000
	.db %11111100

RadarBox:
	.db %11110000
	.db %10010000
	.db %11110000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000


#include "cncsbspr.asm"

#include "cncmpspr.asm"

ControlPic:
; compressed picture made with RLE2PIC
; RLE2PIC by David Phillips <electrum@tfs.net>


 .db $91,$00,$0c,$91,$ff,$04,$91,$00,$0c,$80,$00,$00
 .db $01,$91,$00,$0c,$80,$00,$00,$01,$91,$00,$0c,$80
 .db $00,$00,$01,$91,$00,$0c,$80,$00,$00,$01,$91,$00
 .db $0c,$80,$00,$00,$01,$91,$00,$0c,$80,$00,$00,$01
 .db $91,$00,$0c,$80,$00,$00,$01,$91,$00,$0c,$80,$00
 .db $00,$01,$91,$00,$0c,$80,$00,$00,$01,$91,$00,$0c
 .db $80,$00,$00,$01,$91,$00,$0c,$80,$00,$00,$01,$91
 .db $00,$0c,$80,$00,$00,$01,$91,$00,$0c,$80,$00,$00
 .db $01,$91,$00,$0c,$80,$00,$00,$01,$91,$00,$0c,$80
 .db $00,$00,$01,$91,$00,$0c,$80,$00,$00,$01,$91,$00
 .db $0c,$80,$00,$00,$01,$91,$00,$0c,$80,$00,$00,$01
 .db $91,$00,$0c,$80,$00,$00,$01,$91,$00,$0c,$80,$00
 .db $00,$01,$91,$00,$0c,$80,$00,$00,$01,$91,$00,$0c
 .db $91,$ff,$04,$91,$00,$0c,$80,$91,$00,$0f,$ff,$80
 .db $ff,$80,$91,$00,$0c,$80,$9c,$80,$9c,$91,$00,$0c
 .db $80,$94,$80,$94,$91,$00,$0c,$80,$94,$80,$94,$91
 .db $00,$0c,$80,$94,$80,$94,$91,$00,$0c,$80,$94,$80
 .db $94,$91,$00,$0c,$80,$94,$80,$94,$91,$00,$0c,$80
 .db $94,$80,$94,$91,$00,$0c,$ff,$94,$ff,$94,$91,$00
 .db $0c,$80,$94,$80,$94,$91,$00,$0c,$80,$94,$80,$94
 .db $91,$00,$0c,$80,$94,$80,$94,$91,$00,$0c,$80,$94
 .db $80,$94,$91,$00,$0c,$80,$94,$80,$94,$91,$00,$0c
 .db $80,$94,$80,$94,$91,$00,$0c,$80,$94,$80,$94,$91
 .db $00,$0c,$ff,$94,$ff,$94,$91,$00,$0c,$80,$94,$80
 .db $94,$91,$00,$0c,$80,$94,$80,$94,$91,$00,$0c,$80
 .db $94,$80,$94,$91,$00,$0c,$80,$94,$80,$94,$91,$00
 .db $0c,$80,$94,$80,$94,$91,$00,$0c,$80,$94,$80,$94
 .db $91,$00,$0c,$80,$94,$80,$94,$91,$00,$0c,$ff,$94
 .db $ff,$94,$91,$00,$0c,$80,$9c,$80,$9c,$91,$00,$0c
 .db $91,$80,$04,$91,$00,$0c,$80,$9c,$80,$8c,$91,$00
 .db $0c,$80,$94,$80,$90,$91,$00,$0c,$80,$9c,$80,$90
 .db $91,$00,$0c,$80,$90,$80,$90,$91,$00,$0c,$80,$90
 .db $80,$8c,$91,$00,$0c,$ff,$80,$ff,$80,$91,$00,$0c
 .db $80,$91,$00,$0f,$91,$ff,$04,$91,$00,$0c,$80,$88
 .db $80,$88,$91,$00,$0c,$88,$aa,$88,$aa,$91,$00,$0c
 .db $91,$9c,$04,$91,$00,$0c,$aa,$88,$aa,$88,$91,$00
 .db $0c,$88,$80,$88,$80

; 1024 bytes compressed to 437 -- 42% of original.

TitlePic:
; compressed picture made with RLE2PIC
; RLE2PIC by David Phillips <electrum@tfs.net>


 .db $91,$00,$47,$7c,$7e,$91,$00,$08,$0f,$c0,$fc,$07
 .db $c1,$f0,$fe,$7e,$03,$f0,$3c,$1f,$3f,$c0,$00,$00
 .db $7f,$c7,$ff,$87,$e3,$f8,$fe,$7e,$07,$f0,$7f,$1f
 .db $3f,$fc,$00,$00,$ff,$cf,$ff,$cf,$f3,$f8,$fe,$7f
 .db $07,$f8,$7f,$9f,$3f,$fe,$00,$01,$f9,$df,$ff,$ef
 .db $f7,$f8,$fe,$7f,$0f,$fc,$7f,$df,$3f,$ff,$00,$03
 .db $f0,$1f,$83,$ef,$f7,$fc,$fe,$ff,$1f,$7c,$7f,$ff
 .db $3e,$3f,$00,$03,$f8,$5f,$87,$ff,$7f,$7d,$fe,$ef
 .db $1f,$7e,$3f,$ff,$3e,$3f,$00,$01,$ff,$df,$ff,$df
 .db $3f,$7d,$f6,$ef,$3f,$ff,$3e,$7f,$3f,$fe,$00,$00
 .db $ff,$ef,$ff,$9f,$3e,$7d,$f7,$cf,$7f,$ff,$7e,$3f
 .db $3f,$fc,$00,$00,$7f,$c3,$ff,$1f,$1e,$7f,$f7,$cf
 .db $fc,$1f,$fe,$1f,$3f,$f0,$00,$00,$04,$91,$00,$04
 .db $03,$b3,$cf,$80,$91,$00,$0d,$03,$80,$91,$00,$0e
 .db $0b,$a0,$91,$00,$0e,$7f,$f0,$91,$00,$0e,$ff,$f8
 .db $91,$00,$0e,$7c,$91,$00,$0a,$3f,$91,$ff,$05,$9f
 .db $91,$ff,$04,$f8,$91,$00,$05,$5f,$91,$ff,$04,$8f
 .db $91,$ff,$03,$f9,$91,$00,$0a,$fc,$0e,$91,$00,$0d
 .db $01,$fb,$fe,$91,$00,$0e,$ff,$ff,$91,$00,$0e,$3f
 .db $fe,$91,$00,$0e,$3f,$f8,$91,$00,$0e,$7f,$fe,$91
 .db $00,$08,$1f,$e0,$ff,$c3,$f0,$fc,$ff,$ff,$3e,$3f
 .db $3f,$fe,$7f,$f8,$00,$00,$7f,$e3,$ff,$f3,$fc,$fd
 .db $ff,$ff,$91,$3f,$03,$fe,$7f,$fc,$00,$00,$ff,$e7
 .db $ff,$fb,$fc,$fd,$f8,$3f,$3e,$3f,$3f,$00,$7e,$fe
 .db $00,$00,$fc,$2f,$c1,$fb,$fe,$7d,$f8,$3f,$be,$3f
 .db $3f,$fe,$7e,$fe,$00,$01,$f8,$0f,$c0,$fb,$ff,$fd
 .db $f8,$3f,$3e,$3f,$3f,$fe,$7f,$f8,$00,$00,$fe,$6f
 .db $e3,$fb,$f7,$fd,$fc,$7f,$7e,$3f,$3f,$00,$7f,$f0
 .db $00,$00,$ff,$e7,$ff,$f3,$f3,$fd,$ff,$ff,$3f,$ff
 .db $3f,$fe,$7f,$f8,$e0,$00,$3f,$f3,$ff,$e3,$f1,$fc
 .db $ff,$fe,$1f,$fe,$3f,$fe,$7e,$fe,$e0,$00,$0f,$e0
 .db $7f,$03,$f0,$fc,$7f,$fc,$0f,$f8,$3f,$fe,$7e,$7e
 .db $e0,$91,$00,$07,$0f,$f0,$91,$00,$0f,$c0,$91,$00
 .db $0f,$60,$91,$00,$0f,$20,$91,$00,$0f,$30,$91,$00
 .db $ff,$91,$00,$68

; 1024 bytes compressed to 412 -- 40% of original.


; Text

TitleText1:
 .db "by David Phillips",0
TitleText2:
 .db "graphics by Dave Scheltema",0

MainMenuData:
 .db 5
 .db "Command & Conquer 86",0
 .db "Skirmish",0
 .db "Link Battle",0
 .db "Options",0
 .db "Help",0
 .db "Exit",0
 .dw SingleStart,MultiStart,Options,Help,Exit

HelpText:
 .db $ed,0
 .db "/-------------------\\",0
 .db "| Command & Conquer |",0
 .db "|   version 0.01a   |",0
#ifdef __ASMIDE_VER
 .db "|     "
 .db __DATE__ 
 .db "      |",0
#endif
 .db "\\-------------------/",0
 .db " ",0
 .db "Programmer:",0
 .db "  David Phillips",0
 .db "  <electrum@tfs.net>",0
 .db " ",0
 .db "Graphics:",0
 .db "  Dave Scheltema",0
 .db "  <scheltem@",0
 .db "   aaps.k12.mi.us>",0
 .db " ",0
 .db "Story:",0
 .db "  The forces of GDI",0
 .db " and NOD are battling",0
 .db " for control of the",0
 .db " planet.  You must",0
 .db " command your forces",0
 .db " and conquer the",0
 .db " opposition.  To do",0
 .db " this you build a",0
 .db " base, troops, tanks",0
 .db " and other devices of",0
 .db " destruction.",0
 .db " ",0
 .db "Controls:",0
 .db " Arrows = move the",0
 .db "  mouse cursor around",0
 .db " 2nd = click mouse",0
 .db " Exit = quit game",0
 .db " More = pause game",0
 .db " Alpha/Graph = scroll",0
 .db "  the building list",0
 .db " x-Var/Table = scroll",0
 .db "  the unit list",0
 .db " ",0
 .db "Units:           $$$$",0
 .db " Minnigunner      100",0
 .db " Grenadier        160",0
 .db " Bazooka          300",0
 .db " Flamethrower     300",0
 .db " Engineer         500",0
 .db " Commando         900",0
 .db " Hum Vee          350",0
 .db " Recon Bike       500",0
 .db " Light Tank       600",0
 .db " Medium Tank      700",0
 .db " Heavy Tank       900",0
 .db " Mammoth Tank    1300",0
 .db " Rocket Launcher  800",0
 .db " SSM Launcher     700",0
 .db " Flame Tank       800",0
 .db " Harvester       1000",0
 .db " ",0
 .db "Buildings:       $$$$",0
 .db " Construction Yd  N/A",0
 .db " Power Plant      300",0
 .db " Adv Power Plant  700",0
 .db " Barracks         300",0
 .db " Refinery        2000",0
 .db " Guard Tower      500",0
 .db " Adv Guard Tower 1000",0
 .db " Weapons Factory 2000",0
 .db " Comm Center     1000",0
 .db " Turret           600",0
 .db " Repair Pad      1200",0
 .db " Oblisk of Light 1500",0
 .db " Sandbag Wall      50",0
 .db " Concrete Wall    100",0
 .db " Adv Comm Center 2800",0
 .db " Temple of NOD   3000",0
 .db " ",0
 .db "Testers:",0
 .db " Joey Serling",0
 .db " Christian Rodriguez",0
 .db " Zeromus / MogKupo",0
 .db "  your name here...",0
 .db " ",0
 .db "Special thanks:",0
 .db " Jimmy Mardell-",0
 .db "  PutSprite and",0
 .db "  various routines",0
 .db " ",0
 .db " Pascal Bouron-",0
 .db "  link routines",0
 .db " ",0
 .db " Dave W.(BIOSERRORS)-",0
 .db "  original idea for",0
 .db "  the calc version",0
 .db " ",0
 .db "Enjoy!",0
 .db $ed
; .db " ",0
; .db "                     ",0

PlayCountText:
 .db "This has been played ",0
PlayCountText2:
 .db " times.",0

ProgName:
 .db $12,5,"CnC86"

SettingData:

PlayCount:
 .dw 1

SettingDataEnd:

.end
