/*****************************************************************\
|         CnCEdit v0.50 -- editor for Command & Conquer 86        |
|                                                                 |
| Command & Conquer 86 by                                         |   
|   David Phillips <electrum@tfs.net>                             |
|                                                                 |
| Started    : 09/24/98 -- DP                                     |
| Last Update: 12/16/98 -- DP                                     |
|                                                                 |
| Permission is granted to use this code in your own programs.    |
| If you would like to improve on this editor or use it as a base |
| to write your own editor, you are free to do so.  Please do     |
| not release a modified version without our permission, however. |
|                                                                 |
| This should compile under DOS 16 or 32 bit compilers.           |
| You will have to change the graphics primitives to your own     |
| systems as well.  It was designed to be used with 256 color     |
| modes, but could work with any modes without much modification. |
|                                                                 |
| If anyone wants to write a Windows editor, please email me!     |
\*****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>

//======== TYPEDEFS/STRUCTS ===============================================
typedef struct
{
  char  sig[8],         // "**TI86**"
        ext_sig[3],     // EOF, CR, NULL byte (1a, 0a, 00)
        comment[42];    // padded with NULL bytes to len of 42
  short file_len;       // file length minus 0x39
} FileHeader;

typedef struct
{
  short data_ptr,       // points to var_len2 at end of entry
        var_len;        // length of variable
  char  type,           // variable type
        name_len,       // variable name length
        name[8];        // variable name
  short var_len2;       // exactly the same as var_len
} FileEntry;

typedef struct
{
  short len;            // length of the program
  char  token[2];       // for a compiled asm prgm, (8e, 28)
} VarPrgm;

typedef struct
{
  short len;            // length of the string
} VarString;

//======== GLOBAL VARIABLES ===============================================

char numspr;

char sprites[256][8];       // (8 x 8) 8-byte sprites
char map[64][64],           // maps are 64x64
     temp[64][64];

//======== GRAPHICS PRIMITIVES ============================================
#ifdef __WATCOMC__
  #include <graph.h>
  void set_gfx_mode() 
  {
    _setvideomode(_MRES256COLOR);
  }
  void set_text_mode()       
  {
    _setvideomode(_TEXTC80);
  }
  void put_pixel(short x, short y, char c)
  {
    _setcolor(c);
    _setpixel(x, y);
  }
  void draw_box(short x1, short y1, short x2, short y2, char c)
  {
    _setcolor(c);
    _rectangle(_GBORDER, x1, y1, x2, y2);
  }
  // note:  this expects 25 x 40 coords (x & y swapped) and an 8x8 font
  void put_text(short x, short y, char c, char *text)
  {
    _settextcolor(c);
    _settextposition(x, y);
    _outtext(text);
  }
  #define BLACK 0x00
  #define WHITE 0x07
#endif

//======== KEYBOARD FUNCTIONS =============================================
void wait_for_key()
{
  while (!kbhit());
}

void pause()
{
  wait_for_key();
  while (!getch());   // need to be called twice for extended keys
}

//======== MAP FUNCTIONS ==================================================
void new_map()
{
  short x, y;

  for(x = 0; x < 64; x++)
    for(y = 0; y < 64; y++)
      map[x][y] = 0;
}

void load_map_str(char *name)
{
  FILE        *fp;
  FileHeader  header;
  FileEntry   entry;
  VarString   str;
  char        *fname;
  short       x, y;

  strcpy(fname, name);
  strcat(fname, ".86s");

  fp = fopen(fname, "rb");
  if (fp == NULL)
  {
    printf("\nError opening %s.  Map not loaded!\n", fname);
    return;
  }
  else
  {
    fread(&header, sizeof(FileHeader), 1, fp);
    fread(&entry, sizeof(FileEntry), 1, fp);
    fread(&str, sizeof(VarString), 1, fp);

    fread(&temp, 4096, 1, fp);
    for(y = 0; y < 64; y++)
      for(x = 0; x < 64; x++)
        map[y][x] = temp[x][y];

    fclose(fp);
    if (errno != 0)
    {
      printf("\nError reading %s.  Map not loaded!\n", fname);
      return;
    }
  }
}

void draw_map(short sx, short sy)
{
  short x, y;
  
  for(x = 0; x < 14; x++)
    for(y = 0; y < 8; y++)
        draw_sprite((x * 8) + 105, (y * 8) + 1, map[x + sx][y + sy]);
}

void edit_map()
{
  short x = 0, y = 0,   // sprite list cursor
        a = 0, b = 0,   // map cursor
        n = 0, o = 0;   // corner of map
  char  text[20];
  
  set_gfx_mode();
  draw_sprites();
  put_text(12, 14, 8, "CnCEdit v0.50");
  draw_map(0, 0);

  draw_box(104, 0, 217, 65, 1);
  draw_box(1, 101, 10, 110, 2);
  draw_box(105, 1, 112, 8, 2);
  draw_box(39, 23, 56, 40, 1);
  draw_box(263, 23, 280, 40, 1);
        
  sprintf(text, "%2x", 0);
  put_text(7, 6, 2, text);
  sprintf(text, "%2x", map[a + n][b + o]);
  put_text(7, 34, 2, text);
  sprintf(text, "(%2x, %x)", 0, 0);
  put_text(10, 17, 2, text);

  for(;;)
  {
    wait_for_key();
    switch (getch())
    {
      case 27:        // escape
        set_text_mode();
        return;
      case 13:        // enter
      case 32:        // space
        map[a + n][b + o] = (y * 29) + x;
        draw_sprite((a * 8) + 105, (b * 8) + 1, (y * 29) + x);
        draw_sprite_big(264, 24, map[a + n][b + o]);
        sprintf(text, "%2x", map[a + n][b + o]);
        put_text(7, 34, 2, text);
        draw_box((a * 8) + 105, (b * 8) + 1, 
                  (a * 8) + 112, (b * 8) + 8, 2);
        break;
      case 0:
        draw_box((x * 11) + 1, (y * 11) + 101,
                  (x * 11) + 10, (y * 11) + 110, 0);
        draw_box((a * 8) + 105, (b * 8) + 1, 
                  (a * 8) + 112, (b * 8) + 8, 0);
        draw_sprite((a * 8) + 105, (b * 8) + 1, map[a + n][b + o]);
        switch (getch())
        {
          case 72:    // up
            y--;
            if (y < 0)
            {
              if (x > 23)
                y = 7;
              else
                  y = 8;
            }
            break;
          case 80:    // down
            y++;
            if (((y > 7) && (x > 23)) || (y > 8))
              y = 0;
            break;
          case 75:    // left
            x--;
            if (x < 0)
              if (y < 8)
                x = 28;
              else
                  x = 23;
            break;
          case 77:    // right
            x++;
            if ((x > 28) || ((x > 23) && (y == 8)))
              x = 0;
            break;
          case 71:    // home
            b--;
            if (b < 0)
            {
              b = 0;
              if (o > 0)
              {
                o--;
                draw_map(n, o);
              }
            }
            break;
          case 79:    // end
            b++;
            if (b > 7)
            {
              b = 7;
              if (o < 64 - 8)
              {
                o++;
                draw_map(n, o);
              }
            }
            break;
          case 83:    // delete
            a--;
            if (a < 0)
            {
              a = 0;
              if (n > 0)
              {
                n--;
                draw_map(n, o);
              }
            }
            break;
          case 81:    // page down
            a++;
            if (a > 13)
            {
              a = 13;
              if (n < 64 - 14)
              {
                n++;
                draw_map(n, o);
              }
            }
            break;
        }
        draw_sprite_big(40, 24, (y * 29) + x);
        sprintf(text, "%2x", (y * 29) + x);
        put_text(7, 6, 2, text);
        draw_sprite_big(264, 24, map[a + n][b + o]);
        sprintf(text, "%2x", map[a + n][b + o]);
        put_text(7, 34, 2, text);
        draw_box((x * 11) + 1, (y * 11) + 101,
                  (x * 11) + 10, (y * 11) + 110, 2);
        draw_box((a * 8) + 105, (b * 8) + 1, 
                  (a * 8) + 112, (b * 8) + 8, 2);
        sprintf(text, "(%2i,%2i)", a + n, b + o);
        put_text(10, 17, 2, text);
    }
  }
}

void save_map_str(char *name)
{
  FILE  *fp;
  short x, y; 
  unsigned short checksum;
  FileHeader header;
  char fname[16];

  strcpy(fname, name);
  strcat(fname, ".86s");
  
  memcpy(header.sig, "**TI86**", 8);
  header.ext_sig[0] = 0x1a;
  header.ext_sig[1] = 0x0a;
  header.ext_sig[2] = 0x00;
  memset(header.comment, 0, 42);
  memcpy(header.comment, "cnc86map", 8); //CnCEdit map for CnC86
  header.file_len = 2 + sizeof(FileEntry) + 4096;

  fp = fopen(fname, "wb");
  if (fp == NULL)
    printf("\nError opening %s.  Map not saved!\n", fname);
  else
  {
    fwrite(&header, sizeof(header), 1, fp);

    checksum = 0;
    fputc_sum(4 + strlen(name), fp, &checksum);
    fputc_sum(0, fp, &checksum);
    fputc_sum((4096 + 2) % 256, fp, &checksum);
    fputc_sum((4096 + 2) >> 8, fp, &checksum);
    fputc_sum(0x0c, fp, &checksum);
    fputc_sum(strlen(name), fp, &checksum);
    for(x = 0; x < strlen(name); x++)
      fputc_sum(name[x], fp, &checksum);
    for(x = 0; x < 8 - strlen(name); x++)
      fputc_sum('\0', fp, &checksum);
    fputc_sum((4096 + 2) % 256, fp, &checksum);
    fputc_sum((4096 + 2) >> 8, fp, &checksum);
    fputc_sum(4096 % 256, fp, &checksum);
    fputc_sum(4096 >> 8, fp, &checksum);

    for(y = 0; y < 64; y++)
      for(x = 0; x < 64; x++)
        fputc_sum(map[x][y], fp, &checksum);
    fwrite(&checksum, 2, 1, fp);

    fclose(fp);
    if (errno != 0)
    {
      printf("\nError writting %s.  Map not saved!\n", fname);
      return;
    }
    printf("Map saved as %s\n", fname);
  }
}

fputc_sum(char c, FILE *fp, unsigned short *sum)
{
  *sum += c;
  fputc(c, fp);
}

//======== SPRITE FUNCTIONS ===============================================
void read_sprites(char *fname)
{
  FILE        *fp;
  short       spr_off;
  FileHeader  header;
  FileEntry   entry;
  VarPrgm     prgm;

  fp = fopen(fname, "rb");
  if (fp == NULL)
  {
    printf("Cannon open file %s -- make sure it is in\n", fname);
    printf("the same directory (folder) that you ran CnCEdit from.\n");
    exit(1);
  }
  else
  {
    fread(&header, sizeof(FileHeader), 1, fp);
    fread(&entry, sizeof(FileEntry), 1, fp);
    fread(&prgm, sizeof(VarPrgm), 1, fp);
      
    fseek(fp, 8, SEEK_CUR);
    fread(&spr_off, 2, 1, fp);
    fread(&numspr, 1, 1, fp);
    fseek(fp, spr_off - 11, SEEK_CUR);
    fread(&sprites, 8, numspr, fp);

    fclose(fp);
    if (errno != 0)
    {
      printf("Error reading %s -- file is corrupt!\n", fname);
      exit(1);
    }
  }
}

void draw_sprite(short x, short y, char spr)
{
  short row,
        col;
  char  bitmask;

  for(row = y; row < y + 8; row++)
  {
    bitmask = 0x80;
    for(col = x; col < x + 8; col++)
    {
      if ((sprites[spr][row - y] & bitmask) / bitmask == 1)
        put_pixel(col, row, WHITE);
      else
        put_pixel(col, row, BLACK);
      bitmask >>= 1;
    }
  }
}

void draw_sprite_big(short x, short y, char spr)
{
  short row,
        col;
  char  bitmask;

  for(row = y; row < y + 16; row += 2)
  {
    bitmask = 0x80;
    for(col = x; col < x + 16; col += 2)
    {
      if ((sprites[spr][(row - y) / 2] & bitmask) / bitmask == 1)
      {
        put_pixel(col, row, WHITE);
        put_pixel(col + 1, row, WHITE);
        put_pixel(col, row + 1, WHITE);
        put_pixel(col + 1, row + 1, WHITE);
      }
      else
      {
        put_pixel(col, row, BLACK);
        put_pixel(col + 1, row, BLACK);
        put_pixel(col, row + 1, BLACK);
        put_pixel(col + 1, row + 1, BLACK);
      }
      bitmask >>= 1;
    }
  }
}

void draw_sprites()
{
  short n, x, y;

  n = 0;
  x = 2;
  y = 102;
  while (n < numspr)
  {
    if (x > 310)
    {
      x = 2;
      y += 11;
    }
    draw_sprite(x, y, n);
    n++;
    x += 11;
  }
  draw_box(0, 100, 319, 199, 1);
}

//======== MAIN() =========================================================
void main(int argc, const char *argv[])
{
  char name[16];
  
  printf("\nCnCEdit v0.50 -- Command & Conquer 86 map editor\n\n");
  
  read_sprites("cnc86.86p");

  if (argc == 2)
    strcpy(name, argv[1]);
  else
  {
    printf("usage:  cncedit [mapname]\n\n");
    printf("note: mapname should NOT have an extension!\n");
    printf("      the name will be the string name on the calc\n");
    printf("\nmapname> ");
    gets(name);
  }

  new_map();
  load_map_str(name);
  
  edit_map();
  
  save_map_str(name);
}

//======== OLD CODE =======================================================
/*
void save_map_asm(char *name)
{
  FILE  *fp;
  short x, y;
  char fname[16];

  strcpy(fname, name);
  strcat(fname, ".asm");

  printf("Saving map as %s...", fname);
  fp = fopen(fname, "w");
  if (fp == NULL)
    printf("\nError opening %s.  Map not saved!\n", fname);
  else
  {
    fprintf(fp, "; created with CnCEdit\n\n");
    for(y = 0; y < 64; y++)    
    {
      fprintf(fp, " .db ");
      for(x = 0; x < 31; x++)
        fprintf(fp, "$%02x,", map[x][y]);
      fprintf(fp, "$%02x\n", map[x][y]);
      fprintf(fp, " .db ");
      for(x = 32; x < 63; x++)
        fprintf(fp, "$%02x,", map[x][y]);
      fprintf(fp, "$%02x\n", map[x][y]);
    }
    fclose(fp);
    if (errno != 0)
    {
      printf("\nError writting %s.  Map not saved!\n", fname);
      return;
    }
    printf("done!\n");
  }
}

void load_map_asm(char *name)
{
  FILE  *fp;
  short x, y;
  char line[512],
       temp[3],
       c,
       fname[16];

  strcpy(fname, name);
  strcat(fname, ".asm");
  
  printf("Loading map %s...", fname);
  fp = fopen(fname, "r");
  if (fp == NULL)
  {
    printf("\nError opening %s.  Map not loaded!\n", fname);
    return;
  }
  else
  {
    fgets(line, 512, fp);
    fgets(line, 512, fp);
    for(y = 0; y < 64; y++)
    {
      fgets(line, 512, fp);
      for(x = 0; x < 32; x++)
      {
        memcpy(temp, &line[6 + (x * 4)], 2);
        temp[2] = '\0';
        sscanf(temp, "%x", &c);
        map[x][y] = c;
      }
      fgets(line, 512, fp);
      for(x = 32; x < 64; x++)
      {
        memcpy(temp, &line[6 + ((x - 32) * 4)], 2);
        temp[2] = '\0';
        sscanf(temp, "%x", &c);
        map[x][y] = c;
      }
    }
    fclose(fp);
    if (errno != 0)
    {
      printf("\nError reading %s.  Map not loaded!\n", fname);
      return;
    }
    printf("done!\n");
  }
}
*/
