#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char map[64][64];           // maps are 64x64

typedef struct
{
  char  sig[8],         // "**TI86**"
        ext_sig[3],     // EOF, CR, NULL byte (1a, 0a, 00)
        comment[42];    // padded with NULL bytes to len of 42
  short file_len;       // file length minus 0x39
} FileHeader;

typedef struct
{
  short data_ptr,       // points to var_len2 at end of entry
        var_len;        // length of variable
  char  type,           // variable type
        name_len,       // variable name length
        name[8];        // variable name
  short var_len2;       // exactly the same as var_len
} FileEntry;

typedef struct
{
  short len;            // length of the program
  char  token[2];       // for a compiled asm prgm, (8e, 28)
} VarPrgm;

typedef struct
{
  short len;            // length of the string
} VarString;

void save_map_str(char *name)
{
  FILE  *fp;
  short x, y; 
  unsigned short checksum;
  char *p;
  FileHeader header;
  char fname[16];

  strcpy(fname, name);
  strcat(fname, ".86s");
  
  memcpy(header.sig, "**TI86**", 8);
  header.ext_sig[0] = 0x1a;
  header.ext_sig[1] = 0x0a;
  header.ext_sig[2] = 0x00;
  memset(header.comment, 0, 42);
  memcpy(header.comment, "cnc86map", 8); //CnCEdit map for CnC86
  header.file_len = 10 + strlen(name) + 4096;

  fp = fopen(fname, "wb");
  if (fp == NULL)
    printf("\nError opening %s.  Map not saved!\n", fname);
  else
  {
    fwrite(&header, sizeof(header), 1, fp);

    checksum = 0;
    fputc_sum(4 + strlen(name), fp, &checksum);
    fputc_sum(0, fp, &checksum);
    fputc_sum((4096 + 2) % 256, fp, &checksum);
    fputc_sum((4096 + 2) >> 8, fp, &checksum);
    fputc_sum(0x0c, fp, &checksum);
    fputc_sum(strlen(name), fp, &checksum);
    for(x = 0; x < strlen(name); x++)
      fputc_sum(name[x], fp, &checksum);
    for(x = 0; x < 8 - strlen(name); x++)
      fputc_sum('\0', fp, &checksum);
    fputc_sum((4096 + 2) % 256, fp, &checksum);
    fputc_sum((4096 + 2) >> 8, fp, &checksum);
    fputc_sum(4096 % 256, fp, &checksum);
    fputc_sum(4096 >> 8, fp, &checksum);

    for(y = 0; y < 64; y++)
      for(x = 0; x < 64; x++)
        fputc_sum(map[y][x], fp, &checksum);
    fwrite(&checksum, 2, 1, fp);

    fclose(fp);
    if (errno != 0)
    {
      printf("\nError writting %s.  Map not saved!\n", fname);
      return;
    }
    printf("Map saved as %s\n", fname);
  }
}

fputc_sum(char c, FILE *fp, unsigned short *sum)
{
  *sum += c;
  fputc(c, fp);
}

void main(int argc, const char *argv[])
{
  int  x, y, tiles;
  char name[12];

  printf("\nRandmap -- map generator for C&C86\n");

  if (argc == 3)
  {
    strcpy(name, argv[1]);
    sscanf(argv[2], "%i", &tiles);
  }
  else
  {
    printf("\nusage:  randmap [name] [tiles]\n");
    printf("\nnote: mapname should NOT have an extension!\n");
    printf("      the name will be the string name on the calc\n");
    printf("\nname> ");
    gets(name);
    printf("tiles> ");
    scanf("%i", &tiles);
  }

  for(y = 0; y < 64; y++)
    for(x = 0; x < 64; x++)
      map[y][x] = rand() % tiles;

  save_map_str(name);
}
